Type: Package
Package: ARUtools
Title: Management and Processing of Autonomous Recording Unit (ARU)
        Data
Version: 0.7.3
Authors@R: c(
    person("David", "Hope", , "david.hope@ec.gc.ca", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2140-4261")),
    person("Steffi", "LaZerte", , "sel@steffilazerte.ca", role = "aut",
           comment = c(ORCID = "0000-0002-7690-8360")),
    person("Government of Canada", role = c("cph", "fnd"))
  )
Description: Parse Autonomous Recording Unit (ARU) data and for sub-sampling recordings. 
    Extract Metadata from your recordings, select a subset of recordings for 
    interpretation, and prepare files for processing on the 
    'WildTrax' <https://wildtrax.ca/> platform. Read and process metadata 
    from recordings collected using the SongMeter and BAR-LT types of ARUs. 
License: MIT + file LICENSE
URL: https://arutools.github.io/ARUtools/,
        https://github.com/ARUtools/ARUtools
BugReports: https://github.com/ARUtools/ARUtools/issues
Depends: R (>= 4.1.0)
Imports: dplyr, fs (>= 1.6.1), glue, here, hms (>= 1.1.2), lifecycle,
        lubridate (>= 1.9.3), lutz, parzer (>= 0.4.4), purrr, readr,
        rlang (>= 0.4), seewave (>= 2.2.3), sf, spsurvey (>= 5.0.1),
        stringr, suncalc (>= 0.5.0), tidyr, units, withr
Suggests: covr, ggplot2, jsonlite, knitr, parallel, patchwork, readxl
        (>= 1.4.2), rmarkdown, rstudioapi, soundecology, testthat (>=
        3.0.0), tuneR, vdiffr (>= 1.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-07-28 17:29:18 UTC; HopeD
Author: David Hope [aut, cre] (ORCID: <https://orcid.org/0000-0002-2140-4261>),
  Steffi LaZerte [aut] (ORCID: <https://orcid.org/0000-0002-7690-8360>),
  Government of Canada [cph, fnd]
Maintainer: David Hope <david.hope@ec.gc.ca>
Repository: CRAN
Date/Publication: 2025-07-28 18:20:18 UTC
Built: R 4.6.0; ; 2026-04-26 16:13:04 UTC; unix
