library(hexSticker)

sticker("InkedLOGO_LI.jpg", package="ARUtools",  p_size=20, s_x=1, s_y=.75, s_width=.6,
        p_family = 'gochi',
        filename="inst/figures/ARUtools.png", h_fill = 'white', p_color = 'black',
        h_color = 'darkblue'
          )
